﻿#Requires –Version 3

Param 
      (
          [string]$vmName =$(throw '- Der Parameter -vmName wird benoetigt!') ,
          [string]$configfile
      )


$Start=Get-Date
write-host "     ************************************************************     " -ForegroundColor DarkGreen
write-host "     *                                                          *     " -ForegroundColor DarkGreen
write-host "     *   AI2 Installation der WSUS Rolle  by Thorsten Raucamp   *     " -ForegroundColor DarkGreen
write-host "     *                                                          *     " -ForegroundColor DarkGreen
write-host "     *                  Version 0.95 2014-05-15                 *     " -ForegroundColor DarkGreen
write-host "     *                                                          *     " -ForegroundColor DarkGreen
write-host "     ************************************************************     " -ForegroundColor DarkGreen
Write-Host ""
Write-Host ""
Write-Host ""
write-host "Konfigurie VM $VMName am: " -NoNewline
Get-Date
$Host.UI.RawUI.WindowTitle = "Configure WSUS -- $vmName --"
if (!(Test-Admin)) {
                    $powershell = [System.Diagnostics.Process]::GetCurrentProcess()
                    $psi = New-Object System.Diagnostics.ProcessStartInfo $powerShell.Path
                    $script = $MyInvocation.MyCommand.Path
                    $prm = $script
                    foreach($a in $args) {$prm += ' ' + $a}
                    $psi.Arguments = $prm
                    $psi.Verb = "runas"
                    #$psi
                    [System.Diagnostics.Process]::Start($psi) | Out-Null
                    return;
                    # Neustart des Skriptes mit Adminrechten, sollte das aktuelle Skript nicht mit Administratorrechten 
                    # gestartet worden sein. Übernommen von der c't www.ct.de/1017144
                   }
if([string]::IsNullOrEmpty($configfile)) {$configfile=(Get-ItemProperty -path HKLM:\SOFTWARE\AI2).Configfile}
if (!(Test-Path $configfile)) {Write-Host "Bitte starten Sie das Skript erneut mit dem Parameter -configfile und der entsprechenden Pfadangabe"}
set-ItemProperty -Path HKLM:\SOFTWARE\AI2 -name Configfile -Value $configfile
[xml]$config = Get-Content $configfile
# Ende Header
$logfile=$config.cfg.log +$vmName+"-WSUS.txt"
$ErrorActionPreference="SilentlyContinue"
Stop-Transcript | Out-Null
$ErrorActionPreference="Continue"

Start-Transcript -Path $logfile -Append
#################################################################################################################################################

# Ende Header



$Domain=$config.CFG.Domain 
$DomainAdmin = New-Object System.Management.Automation.PSCredential ($Domain.Name + "\"+$Domain.Admin), (ConvertTo-SecureString $Domain.AdminPW -AsPlainText -Force)

 $srv =$config.CFG.VMs.vm | where {$_.name -match $VMName}
 $srvIP=$srv.srvip
write-host "Überprüfe die Voraussetzungen für den Server $srvIP ..."

        wait-forHost $srvIP

        Write-Host "Überprüfe, ob $srvIP Mitglied in der Domäne ist" -BackgroundColor DarkGreen -ForegroundColor Yellow
        check-DomainMember  $srvIP
$wsus=$config.CFG.Rollen.WSUS      

  Invoke-Command  -ComputerName $srvIP -Credential $DomainAdmin  -Scriptblock {

                    if (!((get-windowsfeature -name UpdateServices-Services).installed))
                            {   Write-Host "Installiere Windowsfeatures für WSUS" -ForegroundColor Cyan
                                Install-WindowsFeature -name UpdateServices-WidDB, UpdateServices-Services, UpdateServices-RSAT, UpdateServices-API,UpdateServices-UI -IncludeManagementTools
                            }
                    if (((Get-Item -Path HKLM:\SOFTWARE\AI2).GetSubKeyNames()-notcontains"WSUS"))
                                {write-host "Konfiguriere WSUS-Dienst..."
                                 # inspiriert durch http://p0w3rsh3ll.wordpress.com/2013/02/05/wsus-on-windows-server-2012-core-from-scratch/
                                Out-File -FilePath c:\log\ps.txt -Append -InputObject "Starte WSUS-Konfiguration"
                                get-date  
                                $wsusWsusDir=$using:wsus.WsusDir
                                md $wsusWsusDir
                                Write-Host "Post-Install des WSUS wird ausgeführt, dieses kann einige Zeit dauern..."
                                & 'C:\Program Files\Update Services\Tools\WsusUtil.exe' postinstall content_dir=$wsusWsusDir
                                Write-Host "Post-Install wurde ausgeführt! " -NoNewline -ForegroundColor Green
                                Get-Date
                                Write-Host "WSUS wird jetzt konfiguriert.." 
                                Start-Service wuauserv 
                                sleep -Seconds 30
                                import-module updateservices #|Out-File -FilePath c:\log\ps.txt -Append 

                                if([string]::IsNullOrEmpty($using:WSUS.SyncServer))
                                    {Set-WsusServerSynchronization -SyncFromMU |Out-File -FilePath c:\log\ps.txt -Append }
                                else {Set-WsusServerSynchronization -useServerName $using:WSUS.SyncServer -Portnumber $using:WSUS.SyncPort }
                                $s=get-wsusserver
                                $s
                                $c=$s.GetConfiguration()
                                if(!([string]::IsNullOrEmpty($using:WSUS.ProxyServer)))
                                    {$c.ProxyName=$using:WSUS.ProxyServer}
                                if(!([string]::IsNullOrEmpty($using:WSUS.ProxyPort)))
                                    {$c.ProxyServerPort=$using:WSUS.ProxyPort}
                                if(!([string]::IsNullOrEmpty($using:WSUS.ProxyServer)))
                                    {$c.UseProxy=$true}
                                else {$c.UseProxy=$false}
                                $c.DownloadExpressPackages=$using:WSUS.DownloadExpressPackages
                                $c.AllUpdateLanguagesEnabled = $false        
                                $c.AllUpdateLanguagesDssEnabled = $true
                                $c.SetEnabledUpdateLanguages("de")    
                                $c.hostBinariesOnMicrosoftUpdate=$false  
                                $c.Save()
                                $s.getconfiguration()
                                (Get-WsusServer).GetConfiguration().GetEnabledUpdateLanguages()   
                                
                                $sub=$s.GetSubscription()
                                $sub.StartSynchronizationForCategoryOnly()
                                $sub
                                Write-Host Konfiguration ist abeschlossen! Beginne nun mit der Synchronisation 
                                Write-Host Dieses kann einige Zeit dauern... 
                                while ($sub.GetSynchronizationStatus() -ne 'NotProcessing') {Write-Host waiting on sync; Start-Sleep -Seconds 5} 
                                Write-Host Die Synchronisation wurde beendt.
                                # Set-WsusServerSynchronization -UssServerName <String> [-PortNumber <Int32> ] [-Replica] [-UpdateServer <IUpdateServer> ] [-UseSsl] [-Confirm] [-WhatIf] [ <CommonParameters>]
                                $products=($using:WSUS.Produkte).split(',')
                                $products
                                Get-WsusProduct|where-Object{$_.Product.Title -in ( $products)}|Set-WsusProduct 
                                $Classifications=($using:WSUS.Classifications).split(',')
                                Get-WsusClassification|Where-Object{$_.Classification.Title -in ( $Classifications)}|Set-WsusClassification
                                $Classifications    
                                $sub=$s.GetSubscription()
                                $sub.SynchronizeAutomatically=$using:WSUS.SynchronizeAutomatically
                                $sub.NumberOfSynchronizationsPerDay=$using:WSUS.NumberOfSynchronizationsPerDay
                                $sub.Save()
                               
                                $sub.StartSynchronization()
                                set-ItemProperty -Path HKLM:\SOFTWARE\AI2 -name "WSUS" -Value $true
                    
                    }
                }
            
write-host "Die Konfiguration des WSUS-Servers ist nun beendet!"
"Ausführungszeit: {0:hh}:{0:mm}:{0:ss}" -f ((get-date)-$Start)
get-date
Stop-Transcript | Out-Null


     
