﻿Param 
      (
      [string]$vmName =$(throw '- Der Parameter -vmName wird benoetigt!') ,
      [string]$Source,
      [string]$SourceSPBinaries,
      [string]$Destination,
      [string]$SPVersion,
      [switch]$noCopy,
      [switch]$noConfig
      )


write-host "     ************************************************************     " -ForegroundColor DarkGreen
write-host "     *                                                          *     " -ForegroundColor DarkGreen
write-host "     *      Skript Prepare-SP für AI2 by Thorsten Raucamp       *     " -ForegroundColor DarkGreen
write-host "     *                                                          *     " -ForegroundColor DarkGreen
write-host "     *                  Version 0.95 2014-05-15                 *     " -ForegroundColor DarkGreen
write-host "     *                                                          *     " -ForegroundColor DarkGreen
write-host "     ************************************************************     " -ForegroundColor DarkGreen
Write-Host ""
Write-Host ""

$Start=Get-Date
if([string]::IsNullOrEmpty($configfile)) {$configfile=(Get-ItemProperty -path HKLM:\SOFTWARE\AI2).Configfile}
[xml]$config = Get-Content $configfile


$lang=Import-Csv ((Split-Path $configfile -Parent)+"\translations.csv")
$LID=(((Get-WmiObject Win32_OperatingSystem))).OSLanguage
$ConfirmPreference = "none"
$logfile=$config.cfg.log +"prepareSharePointFiles.txt"
$ErrorActionPreference="SilentlyContinue"
Stop-Transcript | Out-Null
$ErrorActionPreference="Continue"

Start-Transcript -Path $logfile -Append
$Host.UI.RawUI.WindowTitle = "-- Prepare-SP-Files -- AI2 --"
Write-Host "Configfile: " -NoNewline -ForegroundColor Yellow
Write-Host "$configfile" -ForegroundColor Green
$ScriptDir=(Get-ItemProperty HKLM:\SOFTWARE\AI2).scriptdir
$IsoDir=$config.CFG.HOST.ISO
if([string]::IsNullOrEmpty($Source)){$Source= $config.CFG.HOST.DL+"\SPinst"}
if (!(Test-Path $Source)) 
    { Write-Host "Das angegebene Verzeichnis $Source existiert nicht. Bitte überprüfen !" -ForegroundColor Red
      Break
    }

if([string]::IsNullOrEmpty($SPVersion)){$SPVersion = "2013"}
$SPvNode="SP"
$SPvNode=$SPvNode + $SPVersion

$confignode=$config.CFG.Rollen.SelectSingleNode($SPvNode)


if([string]::IsNullOrEmpty($SourceSPBinaries)){$SourceSPBinaries=($confignode.Source)}
if (!(Test-Path $Source)) 
    { Write-Host "Die angegebene Quelle $Source für SharePoint-Binaries existiert nicht. Bitte überprüfen !" -ForegroundColor Red
      Break
    }

if([string]::IsNullOrEmpty($Destination)){$Destination= ($config.CFG.HOST.DL)}
if (!(Test-Path $Destination)) {md $Destination}                                      
if (!(Test-Path $Destination)) {Write-Host "Das angegebene Ziel $Destination für SharePoint-Binaries kann nicht angelegt werden. Bitte überprüfen !" -ForegroundColor Red
      Break
    }                                
$fdest = $Destination+"\$SPvNode"


#########################################################################################################################################
#########################################################################################################################################
#                                                   Kopieren der Binaries und Prerequisites                                             #
#########################################################################################################################################
#########################################################################################################################################
if (!($noCopy))
    {

        if (!(Test-Path $Destination)) {md $Destination}
        if (!(Test-Path $fdest)) {md $fdest}

        Write-Host "Kopiere Installationsdateien für SharePoint"
        $ftdl=  $Source+'\autospinstaller.zip' 
        If (Test-Path ($ftdl))  {Expand-Zip -InputFile $ftdl -DestinationFolder $fdest}
        $fdest = $Destination+"\$SPvNode"
        $ftdl=  (split-path -Parent $configfile)
        If (Test-Path($ftdl+'\AutoSPInstallerFunctionsCustom.ps1')) {robocopy.exe $ftdl ($fdest+"\SP\AutoSPInstaller") "AutoSPInstallerFunctionsCustom.ps1" /np /ns /nc /ndl /np /nfl  | Out-Host}
        If (Test-Path($ftdl+'\SPTranslations.CSV')) {robocopy.exe $ftdl ($fdest+"\SP\AutoSPInstaller") "SPTranslations.CSV" /np /ns /nc /ndl /np /nfl  | Out-Host} 
        $ftdl=$Source+'\autospinstallerGUI.zip' 
        If (Test-Path ($ftdl)) 
                {Expand-Zip -InputFile $ftdl -DestinationFolder ($fdest)
                 write-host "$ftdl OK!" -foregroundcolor cyan }
        $ftdl=$Source+'\PDFiFilter64installer.zip' 
        If (Test-Path($ftdl))
                { #pfad erstellen
                  $xdest=$fdest+"\SP\$SPVersion\PDF\"
                  if (!(Test-Path $xdest)) {md $xdest}
                  Expand-Zip -InputFile $ftdl -DestinationFolder ($xdest)
                  write-host "$ftdl OK!" -foregroundcolor cyan
                }
        $ftdl=$Source
        If (Test-Path($ftdl+'\AdobePDF.png')) 
                { robocopy.exe $ftdl ($fdest+"\sp\$SPVersion\PDF") "adobepdf.png" /np /ns /nc /ndl /np /nfl | Out-Host
                  write-host "$ftdl wurde kopiert" -foregroundcolor cyan}
        $ftdl=  $Source+'\PreRequisites'
        If (Test-Path($ftdl)) {robocopy.exe $ftdl ($fdest+"\sp\$SPVersion\SharePoint\PrerequisiteInstallerFiles") /e /np /ns /nc /ndl /np /nfl | Out-Host}
        $ftdl=  $Source+'\LanguagePacks'
        If (Test-Path($ftdl)) {robocopy.exe $ftdl ($fdest+"\sp\$SPVersion\SharePoint\LanguagePacks") /e /np /ns /nc /ndl /np /nfl | Out-Host}
        $ftdl=  $confignode.files+'\Update'
        If (Test-Path($ftdl)){robocopy.exe $ftdl ($fdest+"\sp\$SPVersion\SharePoint\Updates") /e /np /ns /nc /ndl /np /nfl /J | Out-Host}
        if (!$fdest+"\sp\$SPVersion\SharePoint\PrerequisiteInstallerFiles\sxs") {md $fdest+"\sp\$SPVersion\SharePoint\PrerequisiteInstallerFiles\sxs"}
        If (!([string]::IsNullOrEmpty($SourceSPBinaries))) # Check ob eine Quelle für SP-Binaries existiert
                                    {
                                     if ( [System.IO.Path]::GetExtension($SourceSPBinaries) -eq ".ISO")
                                       {# Es wurde ein Image angegeben.
                                      $drive= mount-ISO -Path $SourceSPBinaries
                                       robocopy.exe $drive ($fdest+"\sp\$SPVersion\SharePoint") /e /np /ns /nc /nfl /ndl /J | Out-Host
                                       Dismount-DiskImage -ImagePath $SourceSPBinaries
                                       }
                                     elseif (!([string]::IsNullOrEmpty([System.IO.Path]::GetExtension($SourceSPBinaries))))
                                       {#Es wurde eine Datei angegeben
                                                $cp= $ConfirmPreference 
                                                $ConfirmPreference = "none"
                                                Write-Host "Extrahiere SharePoint-Dateien!"
                                                $destpath= $fdest+"\sp\$SPVersion\SharePoint"
                                                start $SourceSPBinaries " /passive /quiet  /extract:$destpath" -Wait | Out-Host # Extrahiere diese Datei ins Zielverzeichnis
                                                $ConfirmPreference = $cp
                                           }
                                     else
                                       {#Es wurde ein Verzeichnis angegeben
                                         if (Test-Path $SourceSPBinaries)
                                           { #ist existent
                                             robocopy.exe $SourceSPBinaries ("sp\$fdest")  /e /np /ns /nc /ndl /np /nfl /J | Out-Host #Kopiere das Ganze Verzeichnis ins Ziel
                                            }
                                        }
                                    }
        else
            { 
               If (Test-Path($ftdl+'\sharepoint.exe'))
                    { Write-Host "Extrahiere SharePoint-Dateien!"
                      $destpath= $fdest+"\$SPVersion\SharePoint"
                      start ($ftdl+'\sharepoint.exe') " /passive /quiet /extract:$destpath" -Wait | Out-Host # Extrahiere diese Datei ins Zielverzeichnis
                    } 

            }
        Write-Host "Dateien wurden kopiert!"
        Remove-ReadOnlyAttribute $fdest
}                             
#########################################################################################################################################
#########################################################################################################################################                        
#                                                  Modifizieren der AutoSPInstallerInput.xml                                            #
#########################################################################################################################################
#########################################################################################################################################
if (!($noConfig))
    {
        $SPsrv=$vmname
        $sqlsrv=$confignode.DBServer
        $inputFile=((split-path -Parent $configfile) + "\autoSpInstallerInput.xml")
        $SVC=import-Csv   ((split-path -Parent $configfile) + "\svc.csv")

        [xml]$xmlinput = (Get-Content $inputFile) -replace ("localhost", $SPsrv)


        $domain=$config.cfg.Domain.NetBIOSName
        # Ersetzen der Werte in der autospinstaller.xml
        $xmlinput.Configuration.Install.OfflineInstall="true"
        $xmlinput.Configuration.Install.SKU=$confignode.sku
        $xmlinput.configuration.Environment=$confignode.Environment
        $xmlinput.configuration.Install.DataDir=$confignode.DataDir 
        $xmlinput.configuration.Farm.Services.SMTP.Install=$confignode.SMTPInstall
        $xmlinput.configuration.Farm.Logging.ULSLogs.LogLocation =$confignode.LogLocation
        $xmlinput.configuration.Farm.Logging.UsageLogs.UsageLogDir=$confignode.UsageLogDir

        #AutoAdminLogon
        $Admin=$svc | where-object {$_.Diensttyp -eq "SP_Admin"}
        $xmlinput.Configuration.Install.autoadminlogon.enable="true"
        $xmlinput.Configuration.Install.autoadminlogon.Password=$Admin.password

        # FARM
        $SP_Farm=$svc | where-object {$_.Diensttyp -eq "SP_Farm"}
        $xmlinput.Configuration.Farm.Passphrase=$confignode.Passphrase
        $xmlinput.Configuration.Farm.Account.LeaveInLocalAdmins="true"
        $xmlinput.Configuration.Farm.Account.Username =$domain+"\"+$SP_Farm.sam
        $xmlinput.Configuration.Farm.Account.Password =$SP_Farm.Password
        ##$xmlinput.Configuration.Farm.Account.Email =$SP_Farm.UPN
        $xmlinput.Configuration.Farm.CentralAdmin.Port=$confignode.CentralAdminPort

        # Database
        $xmlinput.Configuration.Farm.Database.DBServer=$confignode.DBAlias
        $xmlinput.Configuration.Farm.Database.DBAlias.Create="true"
        $xmlinput.Configuration.Farm.Database.DBAlias.DBInstance=$sqlsrv+ "\"+$confignode.DBInstanz
        # $xmlinput.Configuration.Farm.Database.DBAlias.DBPort=$confignode.DBPort
        $xmlinput.Configuration.Farm.Database.DBPrefix=$confignode.DBPrefix
        # Mail
        $mailIP=($config.CFG.VMs.vm | where {$_.Rollen -match "Mail"}).srvip
        if([string]::IsNullOrEmpty($mailIP))
            {$mailIP=($config.CFG.VMs.vm | where {$_.Rollen -match "Exchange"}).srvip
             if([string]::IsNullOrEmpty($mailIP)){
                       $mailIP=($config.CFG.VMs.vm | where {$_.Rollen -match "CGS"}).srvip
                       if([string]::IsNullOrEmpty($mailIP)){
                            $mailIP=$config.cfg.Network.MailIP
                            if([string]::IsNullOrEmpty($mailIP)){$mailIP=$srvIP}
                           }
                       }
           }
        $xmlinput.Configuration.Farm.Services.OutgoingEmail.SMTPServer=$mailIP
        $xmlinput.Configuration.Farm.Services.OutgoingEmail.EmailAddress=$SP_Farm.uPN
        $xmlinput.Configuration.Farm.Services.OutgoingEmail.ReplyToEmail=$SP_Farm.uPN

        # $xmlinput.Configuration.Farm.ManagedAccounts.ManagedAccount
        $accounts=$xmlinput.Configuration.Farm.ManagedAccounts.ManagedAccount

        $x1=$accounts |where-object {$_.commonname -eq "SPService"}
        $s1=$svc | where-object {$_.Diensttyp -eq $x1.commonname}
        $x1.Username=$domain +"\"+ $s1.sam
        $x1.Password =$s1.Password
        ## PÜ_PortalAppPool und SP_ProfilesAppPool
        $x1=$accounts |where-object {$_.commonname -eq "Portal"}
        $s1=$svc | where-object {$_.Diensttyp -eq $x1.commonname}
        $x1.Username=$domain +"\"+ $s1.sam
        $x1.Password =$s1.Password

        $x1=$accounts |where-object {$_.commonname -eq "MySiteHost"}
        $s1=$svc | where-object {$_.Diensttyp -eq $x1.commonname}
        $x1.Username=$domain +"\"+ $s1.sam
        $x1.Password =$s1.Password

        $x1=$accounts |where-object {$_.commonname -eq "searchservice"}
        $s1=$svc | where-object {$_.Diensttyp -eq $x1.commonname}
        $x1.Username=$domain +"\"+ $s1.sam
        $x1.Password =$s1.Password

        #$xmlinput.Configuration.Farm.ManagedAccounts.ManagedAccount
        $SP_SuperU=$svc | where-object {$_.Diensttyp -eq "SuperUser"}
        $SP_SuperR=$svc | where-object {$_.Diensttyp -eq "SuperReader"}
    
        # $xmlinput.Configuration.Farm.ObjectCacheAccounts
        $xmlinput.Configuration.Farm.ObjectCacheAccounts.SuperUser=$domain +"\"+$SP_SuperU.sam
        $xmlinput.Configuration.Farm.ObjectCacheAccounts.SuperReader=$domain +"\"+$SP_SuperR.sam


        #$xmlinput.Configuration.WebApplications.WebApplication
        $portal=$xmlinput.Configuration.WebApplications.WebApplication
        $CurPortal=$Portal |where-object {$_.type -eq "Portal"}
        $s1=$svc | where-object {$_.Diensttyp -eq "Portal"}
        #$CurPortal.applicationPoolAccount=$domain +"\"+ $s1.sam
        $curportal.url="http://"+$SPsrv
        $CurPortal.SiteCollections.SiteCollection.siteUrl="http://"+ $SPsrv
        $CurPortal.SiteCollections.SiteCollection.owner=$domain +"\"+ $s1.sam
        $CurPortal.SiteCollections.SiteCollection.Name=$confignode.PortalName
        $CurPortal.SiteCollections.SiteCollection.SearchUrl="http://"+$SPsrv+"/search"
        $CurPortal.SiteCollections.SiteCollection.Template=$confignode.PortalTemplate
        $CurPortal.SiteCollections.SiteCollection.LCID=$confignode.LCID
        $CurPortal.SiteCollections.SiteCollection.Locale=$confignode.Locale
        $CurPortal.SiteCollections.SiteCollection.Time24="true"
        $CurPortal.SiteCollections.SiteCollection.Description = "MyIntraNet"
    
        $s1=$svc | where-object {$_.Diensttyp -eq "SP_Sync"}

        $xmlinput.Configuration.ServiceApps.UserProfileServiceApp.SyncConnectionAccount=$domain +"\"+ $s1.sam
        $xmlinput.Configuration.ServiceApps.UserProfileServiceApp.SyncConnectionAccountPassword=$s1.Password
        $xmlinput.Configuration.ServiceApps.UserProfileServiceApp.Provision="true"
        $xmlinput.Configuration.ServiceApps.UserProfileServiceApp.MySiteHostLocation=""
        $xmlinput.Configuration.ServiceApps.UserProfileServiceApp.MySiteManagedPath=""
        $xmlinput.Configuration.ServiceApps.UserProfileServiceApp.EnableNetBIOSDomainNames="true"
    
        $s1=$svc | where-object {$_.Diensttyp -eq "SP_Search"}
        $xmlinput.Configuration.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications.EnterpriseSearchServiceApplication.ContentAccessAccount=$domain +"\"+ $s1.sam
        $xmlinput.Configuration.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications.EnterpriseSearchServiceApplication.ContentAccessAccountPassword=$s1.Password


        if($confignode.SKU -ne "Foundation"){
            $CurPortal=$Portal |where-object {$_.type -eq "MySiteHost"}
            $s1=$svc | where-object {$_.Diensttyp -eq "MySiteHost"}
            #$CurPortal.applicationPoolAccount=$domain +"\"+ $s1.sam
            $curportal.url="http://"+$SPsrv
            $CurPortal.SiteCollections.SiteCollection.siteUrl="http://"+ $SPsrv+ ":8080"
            $CurPortal.SiteCollections.SiteCollection.owner=$domain +"\"+ $s1.sam
            $CurPortal.SiteCollections.SiteCollection.SearchUrl="http://"+$SPsrv+"/search"
            $CurPortal.SiteCollections.SiteCollection.Template="SPSMSITEHOST#0"
            $CurPortal.SiteCollections.SiteCollection.LCID=$confignode.LCID
            $CurPortal.SiteCollections.SiteCollection.Locale=$confignode.Locale
            $CurPortal.SiteCollections.SiteCollection.Time24="true"
            $xmlinput.Configuration.Install.PIDKey=$confignode.PIDKey 
            $CurPortal.SiteCollections.SiteCollection.Description = "MyIntraNetMySite"
            $s1=$svc | where-object {$_.Diensttyp -eq "SP_ExcelUser"}
            $xmlinput.Configuration.EnterpriseServiceApps.ExcelServices.UnattendedIDUser=$domain +"\"+ $s1.sam
            $xmlinput.Configuration.EnterpriseServiceApps.ExcelServices.UnattendedIDPassword=$s1.Password
            $s1=$svc | where-object {$_.Diensttyp -eq "SP_VisioUser"}
            $xmlinput.Configuration.EnterpriseServiceApps.VisioService.UnattendedIDUser=$domain +"\"+ $s1.sam
            $xmlinput.Configuration.EnterpriseServiceApps.VisioService.UnattendedIDPassword=$s1.Password
            $s1=$svc | where-object {$_.Diensttyp -eq "SP_PerfPointUser"}
            $xmlinput.Configuration.EnterpriseServiceApps.PerformancePointService.UnattendedIDUser=$domain +"\"+ $s1.sam
            $xmlinput.Configuration.EnterpriseServiceApps.PerformancePointService.UnattendedIDPassword=$s1.Password
    
            $s1=$svc | where-object {$_.Diensttyp -eq "SP_ExcelUser"}
            $xmlinput.Configuration.OfficeWebApps.ExcelService.UnattendedIDUser=$domain +"\"+ $s1.sam
            $xmlinput.Configuration.OfficeWebApps.ExcelService.UnattendedIDPassword=$s1.Password
    
            $s1=$svc | where-object {$_.Diensttyp -eq "SP_ProjectServer"}
            $xmlinput.Configuration.ProjectServer.ServiceApp.ServiceAccount=$domain +"\"+ $s1.sam
            $xmlinput.Configuration.ProjectServer.ServiceApp.ServiceAccountPassword=$s1.Password
            $xmlinput.Configuration.ServiceApps.WordAutomationService.Provision="true"

        }
        else
            {   # Diese Apps sind in der Foundation nicht enthalten
                $xmlinput.Configuration.ServiceApps.SecureStoreService.Provision= "false"
                $xmlinput.Configuration.ServiceApps.PowerPointConversionService.Provision= "false"
                $xmlinput.Configuration.ServiceApps.ManagedMetadataServiceApp.Provision= "false"
                $xmlinput.Configuration.ServiceApps.UserProfileServiceApp.Provision= "false"
                $xmlinput.Configuration.ServiceApps.EnterpriseSearchService.Provision= "false"
                $xmlinput.Configuration.ServiceApps.WorkManagementService.Provision = "false"
                $xmlinput.Configuration.ServiceApps.MachineTranslationService.Provision = "false"
                # $xmlinput.Configuration.ServiceApps.StateService.Provision= "false"
                # $xmlinput.Configuration.ServiceApps.WebAnalyticsService.Provision= "false"
                # $xmlinput.Configuration.ServiceApps.SPUsageService.Provision= "false"
                # $xmlinput.Configuration.ServiceApps.WordAutomationService.Provision= "false"
                # MySites entfernen, da die anscheinend unter Foundation nicht funktionieren
                $type="MySiteHost"
                $xmlinput.Configuration.WebApplications.WebApplication | ? {$type -contains $_.type } | %{$xmlinput.Configuration.WebApplications.RemoveChild($_)}
                #$xmlinput.Configuration.Install.PIDKey="NQTMW-K63MQ-39G6H-B2CH9-FRDWJ"
                $xmlinput.Configuration.Install.PIDKey=""
        }


        #$s1=$svc | where-object {$_.Diensttyp -eq "Searchservice"}
        #$xmlinput.Configuration.ServiceApps.EnterpriseSearchService.Account=$domain +"\"+ $s1.sam
        #$xmlinput.Configuration.ServiceApps.EnterpriseSearchService.Password=$s1.Password
        $xmlinput.Configuration.ServiceApps.BusinessDataConnectivity.Provision="true"
        $xmlinput.Configuration.ServiceApps.AppManagementService.AppDomain= "apps." +$domain+ $DomainSuffix
        $xmlinput.Configuration.ServiceApps.AppManagementService.Database.DBAlias.Create="false"
        $xmlinput.Configuration.AdobePDF.iFilter.Install="true"
        $xmlinput.Configuration.ServiceApps.AppManagementService.Provision="false"
        $xmlinput.Configuration.ServiceApps.SubscriptionSettingsService.Provision="false"
        
        $xmlinput.Configuration.Install.RemoteInstall.Enable   ="true"
        $xmlinput.Save( ($fdest+"\SP\AutoSPInstaller\AutoSPInstallerInput.xml"))
        $xmlinput.Save( (Split-Path $logfile -Parent )+"\$VMName-AutoSpInstallerInput.XML") 
        Write-Host " AutoSPInstallerInput.xml wurde modifiziert und gespeichert."
        Write-Host " Die weitere Bearbeitung kann mit AutoSPInstallerGUI im Verzeichnis $fdest vorgenommen werden " -ForegroundColor Cyan

    }
        
        

"Ausführungszeit: {0:hh}:{0:mm}:{0:ss}" -f ((get-date)-$Start)

Get-Date

Stop-Transcript | Out-Null
write-host "OK"
 