﻿Function Test-NetworkPort
{
  param
  (
    $ComputerName = $env:COMPUTERNAME,

    [Int32[]]
    [Parameter(ValueFromPipeline=$true)]
    $Port = $(135..139 + 443 + 445 +443 + 5980..5990),

    [Int32]
    $Timeout=1000,

    [Switch]
    $AllResults
  )

  Process
  {
    $count = 0
    ForEach ($PortNumber in $Port)
    {
      $count ++
      $percentComplete = $count * 100 / $Port.Count
      Write-Progress -Activity "Scanning on \\$ComputerName" -Status "Port $PortNumber" -PercentComplete $percentComplete
      # in PowerShell 2.0 muss [Ordered] entfernt werden; dann ist die Reihenfolge der Eigenschaften aber zufällig.
      $result = New-Object PSObject -Property ([Ordered]@{Port="$PortNumber";Open=$False;Type='TCP';ComputerName=$ComputerName})
      
      $TCPClient = New-Object System.Net.Sockets.TcpClient 
      $Connection = $TCPClient.BeginConnect($ComputerName, $PortNumber, $null, $null)
    
      try
      {
        if ($Connection.AsyncWaitHandle.WaitOne($Timeout, $false))
        {
          $null = $TCPClient.EndConnect($Connection)
          $result.Open = $true
        }
      } 
      catch {} finally { $TCPClient.Close() }

      $result | Where-Object { $AllResults -or $_.Open }
    }
  }
}

#Test-NetworkPort -ComputerName 192.168.2.24
#Test-NetworkPort -ComputerName 192.168.2.24
#Test-NetworkPort -ComputerName 131.107.0.1
Test-NetworkPort -ComputerName 10.0.0.101
#Test-NetworkPort -ComputerName 10.0.0.5


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
